/*
-----------------------------------------------------------------------------------------------------------

  WindowsFunctions.prg

  All Windows API Functions used by this application and functions that use those APIs exclusivley



  March 2005 / Stu Fraser

-----------------------------------------------------------------------------------------------------------
*/
#include "dll.ch"
#include "Gra.ch"
#include "Xbp.ch"
#include "Common.ch"
#include "Appevent.ch"
#include "Font.ch"
#include "fileio.ch"
#include "set.ch"
#include "bap.ch"
#INCLUDE "dcdialog.CH"

//#include "shfile.ch"        // \WDS\Include

// Defines for SHFile from shellapi.h

// Operations:
#define FO_MOVE                      1 // Move file
#define FO_COPY                      2 // Copy file
#define FO_DELETE                    3 // Delete file
#define FO_RENAME                    4 // Rename file

// Flags:
#define FOF_MULTIDESTFILES           1 // The TO member specifies multiple
                                       // destinations (one for each source file)
                                       // rather than one destination directory
                                       // for all files
#define FOF_CONFIRMMOUSE             2 // Not implemented
#define FOF_SILENT                   4 // Don't create progress/report
#define FOF_RENAMEONCOLLISION        8 // Rename the file if one of the target name
                                       // already exists ("Copy #1 of ..." etc)
#define FOF_NOCONFIRMATION          16 // Don't prompt the user.
#define FOF_WANTMAPPINGHANDLE       32 // Fill in SHFILEOPSTRUCT.hNameMappings
                                       // Must be freed using SHFreeNameMappings
#define FOF_ALLOWUNDO               64 // Move file to recycle bin if possible
#define FOF_FILESONLY              128 // On *.*, do only files
#define FOF_SIMPLEPROGRESS         256 // Don't show names of files
#define FOF_NOCONFIRMMKDIR         512 // Don't confirm making any needed dirs
#define FOF_NOERRORUI             1024 // Don't put up error UI
#define FOF_NOCOPYSECURITYATTRIBS 2048 // Dont copy NT file Security Attributes


//-- defines for WinGetUNC

#define UNIVERSAL_NAME_INFO_LEVEL   0x00000001
#define REMOTE_NAME_INFO_LEVEL      0x00000002
#define NO_ERROR                    0
#define ERROR_BAD_DEVICE            1200
#define ERROR_CONNECTION_UNAVAIL    1201
#define ERROR_EXTENDED_ERROR        1208
#define ERROR_MORE_DATA             234
#define ERROR_NOT_SUPPORTED         50
#define ERROR_NO_NET_OR_BAD_PATH    1203
#define ERROR_NO_NETWORK            1222
#define ERROR_NOT_CONNECTED         2250

#DEFINE MAX_BUFFER_SIZE             500
#DEFINE STRUCTURE_HEADER            0

#define VK_ESCAPE         0x001B
#define KEYEVENTF_KEYUP   0x0002

//-- defines from WINUSER.H
#define SW_HIDE             0
#define SW_SHOWNORMAL       1
#define SW_NORMAL           1
#define SW_SHOWMINIMIZED    2
#define SW_SHOWMAXIMIZED    3
#define SW_MAXIMIZE         3
#define SW_SHOWNOACTIVATE   4
#define SW_SHOW             5
#define SW_MINIMIZE         6
#define SW_SHOWMINNOACTIVE  7
#define SW_SHOWNA           8
#define SW_RESTORE          9
#define SW_SHOWDEFAULT      10
#define SW_MAX              10

MEMVAR aTitles

DLLFUNCTION GetComputerNameA( @cComputerName, nLen)   USING STDCALL FROM KERNEL32.DLL
DLLFUNCTION FindWindowA( @ClassName, @WinName )  USING STDCALL FROM USER32.DLL
DLLFUNCTION GetForegroundWindow()                USING STDCALL FROM USER32.DLL
DLLFUNCTION IsIconic( nHwnd )                    USING STDCALL FROM USER32.DLL
DLLFUNCTION GetLastActivePopup( nHwnd )          USING STDCALL FROM USER32.DLL
DLLFUNCTION ShowWindow( nHwnd, nCmdShow )        USING STDCALL FROM USER32.DLL
DLLFUNCTION BringWindowToTop( nHwnd )            USING STDCALL FROM USER32.DLL
DLLFUNCTION SetForegroundWindow( nHwnd )         USING STDCALL FROM USER32.DLL
DLLFUNCTION GetWindowThreadProcessId( nForgroundHwnd, @nRetProcId ) USING STDCALL FROM USER32.DLL
DLLFUNCTION GetWindowTextA( nHwnd,@cFileName,nLen ) USING STDCALL FROM USER32.DLL
DLLFUNCTION GetWindowModuleFileNameA( nHwnd,@cFileName,nLen ) USING STDCALL FROM USER32.DLL
DLLFUNCTION CreateMutexA( SecAttr, nInitialOwn, cName )  USING STDCALL FROM KERNEL32.DLL
DLLFUNCTION EnumWindows( cProc, nPara ) USING STDCALL FROM USER32.DLL
DLLFUNCTION GetLastError()  USING STDCALL FROM KERNEL32.DLL
DLLFUNCTION GetActiveWindow()  USING STDCALL FROM USER32.DLL
DLLFUNCTION keybd_event(nVirtKey,nScanCode,nFlags,nExtra) USING STDCALL FROM USER32.DLL


/*
-----------------------------------------------------------------------------------------------------------

    WinIsRunning(
        cTitle          Full or Partial Title of Window you are Looking For
        lExact          Exact Match necessary or not (defaults to .F.)
        lSetFocus       Set Focus to the Window if Found (defaults to .F.)
        ) -->           .T. if Window Title found, .F. if Not


        Looks for the Passed Window Title to see if it is running and optionally sets focus to it.
        Useful when the full Window Title is not known.
        Can also do exact match of title if desired.


-----------------------------------------------------------------------------------------------------------
*/


//-------------------------------------------------------------------
    Function WinIsRunning(cTitle,lExact,lSetFocus)
//-------------------------------------------------------------------

Local nFind:=0
Local nHndl:=0
Local lRet:=.f.
Local aAllWin[0]

default lExact to .f.
default lSetFocus to .f.

aAllWin:=AllWindows()

If lExact
    nFind:= AScan(aAllWin,{|a| Upper(cTitle) == Upper(a[1])})       // exact match
Else
    nFind:= AScan(aAllWin,{|a| Upper(cTitle) $ Upper(a[1])})        // found in
Endif

nHndl := IIF( nFind > 0, aAllWin[nFind][2], 0 )
lRet  := ( nHndl > 0 )

If lSetFocus .and. lRet
    WinSetFocus(nHndl)
Endif

aSize(aAllWin,0)
aAllwin:=nil

Return lRet

/*

-----------------------------------------------------------------------------------------------------------

  AllWindows(
                lShow       display array via dc_arrayview.  For debugging only, defaults to .F.
                )-->        multi-dim array of all active windows with the following structure:
                            {
                                {<cWinTitle>,<nWinHandle>}
                            }


  Creates and returns an array of all Top Level Windows that are currently open.

-----------------------------------------------------------------------------------------------------------
*/

//----------------------------------
    FUNCTION AllWindows(lShow)
//---------------------------------
Local cProc
Local aResults[0]

PRIVATE aTitles := {}          // has to be private to scope down for the callback procedure

default lShow to .f.

cProc := BaCallBack("EnumWindow",BA_CB_GENERIC2)   // create callback procedure

EnumWindows( cProc,0 )

if lShow
    dc_arrayview(aTitles) // debug only
endif

aResults:=aClone(aTitles)         // get it into a local

aSize(aTitles,0)                  // kill the private array

aTitles:=nil

RETURN (aResults)

/*
-----------------------------------------------------------------------------------------------------------

  EnumWindow(
                nHandle         Window Handle
                nr              Number of Parameters to Return
                )-->1           dummy return value.

    This is a callback routine used by AllWindows above.
    Adds 1 Window Element to the Private array aTitles declared in AllWindows.
    Works like a code block, used by EnumWindows Routine to populate local array.

-----------------------------------------------------------------------------------------------------------
*/

//-----------------------------------------------------
    FUNCTION EnumWindow( nHandle, nr )
//-----------------------------------------------------

LOCAL cBuf := repl(chr(32),100)
LOCAL nLen := len(cBuf)

nLen := GetWindowTextA( nHandle, @cBuf, nLen )

If nLen > 0

    aAdd( aTitles,{left(cBuf,nLen), nHandle} )

Endif

RETURN 1

/*
-----------------------------------------------------------------------------------------------------------

  WinSetFocus(
                nHwndFocus          handle of window to set focus too
                )-->lFocusSet       Allways returns True unless 0 handle passed

  Sets Focus To and brings to the foreground the window specified by nHwndFocus.

-----------------------------------------------------------------------------------------------------------
*/

//---------------------------------------------
    FUNCTION WinSetFocus( nHwndFocus )
//---------------------------------------------

Local nHwndForeground, nForegroundId
Local nFindId, nHwndLast
Local cWinTitle := Space(250)
Local cWinFile := Space(250)
Local lRet  := .f.

default nHwndFocus to 0



If !(nHwndFocus == 0)

    nHwndForeground := GetForegroundWindow()
    nForeGroundId   := GetWindowThreadProcessId( nHwndForeground, 0 )
    nFindId         := GetWindowThreadProcessId( nHwndFocus, 0 )

    If !nForeGroundId == nFindId .or. !IsIconic( nHwndFocus ) == 0

        nHwndLast := GetLastActivePopup( nHwndFocus )

        If !IsIconic( nHwndLast ) == 0

            ShowWindow( nHwndLast, SW_RESTORE )

        Endif

        BringWindowToTop( nHwndLast )
        SetForegroundWindow( nHwndLast )

    Endif

Endif

Return( .t. )


//----------------------------------------------------------------
    Function WinFileCopy(cSource, cTarget, lOverwrite)
//----------------------------------------------------------------

LOCAL nDll := DllLoad("kernel32.dll")
LOCAL lSuccess := .F.

default cSource to ""
default cTarget to ""
default lOverwrite to .f.

If Empty(cSource) .or. Empty(cTarget)
    MsgBox("PROGRAMMER ERROR!, Source File: "+cSource+" Target File: "+cTarget)
Else
    If nDll > 0
        lSuccess := (DllCall(nDll,DLL_STDCALL,"CopyFileA",cSource,cTarget,If(lOverwrite,0,1))==1)
        DllUnload(nDll)
    EndIf
Endif

RETURN lSuccess


//---------------------------------------------------------------------------------------
    Function WinTempPath()
//---------------------------------------------------------------------------------------
LOCAL nDll
LOCAL nBuffSize := 261
LOCAL cBuffer   := Replicate(chr(0),261)

nDll := DllLoad( "KERNEL32.DLL" )

IF nDll # 0
    DllCall( nDll, DLL_STDCALL, "GetTempPathA", nBuffSize, @cBuffer )
    DllUnload( nDll )
ENDIF

RETURN cBuffer



//---------------------------------------------------------------------------------------
    FUNCTION WinGetUNC(tcMappedPath, tnBufferSize)
//---------------------------------------------------------------------------------------

   local cReturnValue
   local cDrive, cPath
   local cLocalPath, cBuffer, nBufferSize, nResult, cStructureString
   Local nDllHandle := DllLoad( "MPR.DLL" )

   IF nDllHandle = 0
      RETURN Nil
   ENDIF

   IF !EMPTY(tcMappedPath)
      DO CASE
         CASE LEN(tcMappedPath) > 2
            cDrive = LEFT(tcMappedPath, 2)
            cPath  = SUBSTR(tcMappedPath, 3)
         CASE len(tcMappedPath) <= 2
            cDrive = tcMappedPath
            cPath  = ""
      ENDCASE

      nBufferSize = IIF(PCOUNT() = 1 .OR. TYPE('tnBufferSize') # "N" .OR. EMPTY(tnBufferSize), MAX_BUFFER_SIZE, tnBufferSize) + 1
      cLocalPath = cDrive
      cBuffer = space(nBufferSize)

      nResult := Dllcall( nDllHandle, DLL_STDCALL, "WNetGetConnectionA", @cLocalPath, @cBuffer, @nBufferSize)
//       nLastError :=DllCall( "Mpr.Dll", DLL_STDCALL, "WNetGetConnectionA", @cDevice, @cServerName, @nLength )
      DO CASE
         CASE nResult = NO_ERROR                    // string translated sucessfully
              cStructureString = ALLTRIM(SUBSTR(cBuffer, STRUCTURE_HEADER + 1))
              cReturnValue = LEFT(cStructureString, AT(CHR(0), cStructureString) - 1) + cPath
         CASE nResult = ERROR_BAD_DEVICE            // The string pointed to by lpLocalPath is invalid.
              cReturnValue = tcMappedPath
         CASE nResult = ERROR_CONNECTION_UNAVAIL    // There is no current connection to the remote device, but there is a remembered (persistent) connection to it.
              cReturnValue = tcMappedPath
         CASE nResult = ERROR_EXTENDED_ERROR        // A network-specific error occurred. Use the WNetGetLastError function to obtain a description of the error.
              cReturnValue = tcMappedPath
         CASE nResult = ERROR_MORE_DATA             // The buffer pointed to by lpBuffer is too small. The function sets the variable pointed to by lpBufferSize to the required buffer size.
              cReturnValue = tcMappedPath
         CASE nResult = ERROR_NOT_SUPPORTED         // The dwInfoLevel parameter was set to UNIVERSAL_NAME_INFO_LEVEL, but the network provider does not support UNC  names. This function is not supported by any of the network providers.
              cReturnValue = tcMappedPath
         CASE nResult = ERROR_NO_NET_OR_BAD_PATH    // None of the providers recognized this local name as having a connection. However, the network is not available for at least one provider to whom the connection may belong.
              cReturnValue = tcMappedPath
         CASE nResult = ERROR_NO_NETWORK            // There is no network present.
              cReturnValue = tcMappedPath
         CASE nResult = ERROR_NOT_CONNECTED         // The device specified by lpLocalPath is not redirected.
              cReturnValue = tcMappedPath
         OTHERWISE
              cReturnValue = tcMappedPath
      ENDCASE
   ELSE
      cReturnValue = tcMappedPath
   ENDIF
RETURN cReturnValue



/*

   SHFile-Class

   What it does :

      Provides a wrapper for the SHFileOperation-API function

   Which methods are avalaible for the "public" :

      Well, they do just what their names are telling !
      For further explanations see the header of each method

      Copy
      Move
      Rename
      Delete

   SHFile needs the BAP-Library from Alaska to work correctly, so
   be sure that you provide the BAP.DLL with your programs.

   Revision history :

      Current Version 0.7

      13.02.2000  some small fixes in the CheckPath method.
                  Added CreateDir and RemoveDir methods

      07.02.2000  Initial Version, delete is basically working
                  copy is working, see notes in method header !

*/

CLASS ShFile FROM xbpStatic

   VAR aOpStruct, cFileOpStruct
   VAR cSrc, cDest, lMultiDest
   VAR nOperation, nDllHandle
   VAR nFlags, hWnd, lConfirm

   METHOD PreparePath, CheckPath, BuildStruct, ExecCall

   EXPORTED:

   METHOD Init, Create, Destroy
   METHOD Copy, Move, Rename, Delete
   METHOD CreateDir, RemoveDir

ENDCLASS

/*-----------------------------------------------------------------------------

METHOD ShFile:Init

*/
METHOD ShFile:Init

   ::xbpStatic:Init()
   ::type := XBPSTATIC_TYPE_TEXT

RETURN self

/*-----------------------------------------------------------------------------

METHOD ShFile:Create( oPar, oOwn, aPos, aSize, aPresPar )

*/
METHOD ShFile:Create( oPar, oOwn, aPos, aSize, aPresPar )

   // Create invisible Object
   ::xbpStatic:Create( oPar, oOwn, aPos, aSize, aPresPar, .F. )
   ::hWnd := ::GetHwnd()
   ::lMultiDest := .F.

RETURN self

/*-----------------------------------------------------------------------------

METHOD ShFile:Destroy

*/
METHOD ShFile:Destroy

   ::aOpStruct :=  ::cFileOpStruct := ::cSrc := ::cDest := ::lMultiDest := NIL
   ::nOperation := ::nDllHandle := ::nFlags := ::hWnd := ::lConfirm := NIL
   ::xbpStatic:Destroy()

RETURN self


/*-----------------------------------------------------------------------------

METHOD ShFile:Copy( xSrc, xDest, lConfirm )

   xSrc and xDest can be single filenames
   or arrays containing several files

   If xDest is an array, the number of elements must be identical
   to the number in xSrc. Each element in xDest must contain the full
   destination filename including the path.

*/
METHOD ShFile:Copy( xSrc, xDest, lConfirm )

   DEFAULT lConfirm TO .F.

   ::cSrc       := ::PreparePath( xSrc )
   ::cDest      := ::PreparePath( xDest )
   ::nOperation := FO_COPY
   If VALTYPE( xDest ) = "A"
      If LEN( xDest ) > 1
         ::lMultiDest := .T.
      EndIF
   EndIF
   ::lConfirm   := lConfirm
   ::BuildStruct()
   ::ExecCall()

RETURN NIL

/*-----------------------------------------------------------------------------

METHOD ShFile:Move( xSrc, xDest, lConfirm )

   Move files from one location to another, just pass .T. in the third
   parameter if you need a "do you really want to do this?" window.

   xSrc can be a single filename or an array containing several files

*/
METHOD ShFile:Move( xSrc, xDest, lConfirm )

   DEFAULT lConfirm TO .F.

   ::cSrc       := ::PreparePath( xSrc )
   ::cDest      := ::PreparePath( xDest )
   ::nOperation := FO_MOVE
   If VALTYPE( xDest ) = "A"
      If LEN( xDest ) > 1
         ::lMultiDest := .T.
      EndIF
   EndIF
   ::lConfirm   := lConfirm
   ::BuildStruct()
   ::ExecCall()

RETURN NIL

/*-----------------------------------------------------------------------------

METHOD ShFile:Rename(xSrc, xDest, lConfirm )

   Rename files, just pass .T. in the third parameter if you
   need a "do you really want to do this?" window.

   xSrc and xDest can be single filenames
   or arrays containing several files

   If xDest is an array, the number of elements must be identical
   to the number in xSrc. Each element in xDest must contain the full
   destination filename including the path.


*/
METHOD ShFile:Rename(xSrc, xDest, lConfirm )

   DEFAULT lConfirm TO .F.

   ::cSrc       := ::PreparePath( xSrc )
   ::cDest      := ::PreparePath( xDest )
   ::nOperation := FO_RENAME
   If VALTYPE( xDest ) = "A"
      If LEN( xDest ) > 1
         ::lMultiDest := .T.
      EndIF
   EndIF
   ::lConfirm   := lConfirm
   ::BuildStruct()
   ::ExecCall()

RETURN NIL

/*-----------------------------------------------------------------------------

METHOD ShFile:Delete( cFile, lConfirm )

*/
METHOD ShFile:Delete( xFile, lConfirm )

   DEFAULT lConfirm TO .F.

   ::cSrc       := ::PreparePath( xFile )
   ::cDest      := "" + CHR(0)
   ::nOperation := FO_DELETE
   ::lConfirm   := lConfirm
   ::BuildStruct()
   ::ExecCall()

RETURN NIL

/*-----------------------------------------------------------------------------

METHOD ShFile:CreateDir( cDir )

   Create a new directory, cDir can be a complete or relative path but
   you can not create a complete path ( C:\SUB1\SUB2 ) in one step, you
   will have to use

   oSHFile:CreateDir( "C:\SUB1" )
   oSHFile:CreateDir( "C:\SUB1\SUB2" )

*/
METHOD ShFile:CreateDir( cDir )
   LOCAL lRet := .F.

   ::nDllHandle :=DllLoad("KERNEL32.DLL")
   IF ::nDllHandle != 0
      lRet := ( DllCall(::nDllHandle,DLL_STDCALL,"CreateDirectoryA", cDir, 0 ) # 0 )
      DllUnload(::nDllHandle)
   ENDIF

RETURN lRet

/*-----------------------------------------------------------------------------

METHOD ShFile:RemoveDir( cDir )

   Remove a directory, cDir can be a complete or relative path

*/
METHOD ShFile:RemoveDir( cDir )
   LOCAL lRet := .F.

   ::nDllHandle :=DllLoad("KERNEL32.DLL")
   IF ::nDllHandle != 0
      lRet := ( DllCall(::nDllHandle,DLL_STDCALL,"RemoveDirectoryA", cDir, 0 ) # 0 )
      DllUnload(::nDllHandle)
   ENDIF

RETURN lRet


/*-----------------------------------------------------------------------------

METHOD ShFile:PreparePath( xSrc )

   Internal helper method to make the filenames what they are supposed to be,
   also provides support for managing a file-list via an array.

*/
METHOD ShFile:PreparePath( xSrc, lCheckPath )
LOCAL cRet, nI

   DEFAULT lCheckPath TO .T.

   If VALTYPE( xSrc ) = "C"
      xSrc := ::CheckPath( xSrc )
      cRet := TRIM(xSrc) + CHR(0)
   EndIF

   If VALTYPE( xSrc ) = "A"

      cRet := ""

      If lCheckPath
         For nI := 1 TO LEN( xSrc )
            xSrc[nI] := ::CheckPath( xSrc[nI] )
         Next
      EndIF

      AEVAL( xSrc, { | cEl | cRet += TRIM(cEl) + CHR(0) } )

      cRet += CHR(0)

   EndIF

RETURN cRet

/*-----------------------------------------------------------------------------

METHOD ShFile:CheckPath( xSrc )

   Internal helper method to check if a fully qualified path is provided

   Valid paths are :

      C:\File
      \\Computer\Share\File

   Invalid paths are :

      \File
      .\File
      File


*/
METHOD ShFile:CheckPath( xSrc )
LOCAL cRet := xSrc
LOCAL lExact := SET( _SET_EXACT, .F. )

   DO CASE
      CASE xSrc = "\\"                           // UNC-Path supplied, nothing to do

      CASE UPPER(SUBSTR( xSrc, 1, 1 )) $ "ABCDEFGHIJKLMNOPQRSTUVWXYZ" .AND.;
           SUBSTR( xSrc, 2, 2 ) = ":\"           // "C:\"-Type

      CASE xSrc = ".\"

         cRet := CURDRIVE() + ":\" + CURDIR(CURDRIVE()) + SUBSTR(xSrc,2)

      CASE xSrc = "\"

         cRet := CURDRIVE() + ":" + xSrc

      OTHERWISE

         cRet := CURDRIVE() + ":\" + CURDIR(CURDRIVE()) + xSrc

   ENDCASE

   SET( _SET_EXACT, lExact )

RETURN cRet

/*-----------------------------------------------------------------------------

  Create structure string needed for the SHFileOperation call in :ExecCall

*/
METHOD ShFile:BuildStruct
   LOCAL lRet
   LOCAL fAborted  := 0
   LOCAL hNameMaps := 0
   LOCAL cProgress := ""

   IF ::lConfirm
      ::nFlags := FOF_ALLOWUNDO + FOF_RENAMEONCOLLISION
   ELSE
      ::nFlags := FOF_ALLOWUNDO + FOF_SILENT + FOF_NOCONFIRMATION + FOF_RENAMEONCOLLISION
   ENDIF

   If ::lMultiDest
      ::nFlags += FOF_MULTIDESTFILES
   EndIF

   ::lMultiDest := .F.

   ::aOpStruct := BaInit(8)

   lRet  := BaStruct(::aOpStruct,::hWnd)       // Not used
   lRet  := BaStruct(::aOpStruct,::nOperation) // Sub-function number
   lRet  := BaStruct(::aOpStruct,@::cSrc)      // Name of file to delete
   lRet  := BaStruct(::aOpStruct,@::cDest)     // Not used
   lRet  := BaStruct(::aOpStruct,::nFlags)     // How to do it
   lRet  := BaStruct(::aOpStruct,fAborted)     // Not used
   lRet  := BaStruct(::aOpStruct,hNameMaps)    // Not used
   lRet  := BaStruct(::aOpStruct,@cProgress)        // Not used

   ::cFileOpStruct := BaAccess(::aOpStruct)

RETURN NIL


/*-----------------------------------------------------------------------------

METHOD ShFile:ExecCall

   Does the actual DLL-calling

*/
METHOD ShFile:ExecCall

   ::nDllHandle :=DllLoad("SHELL32.DLL")
   IF ::nDllHandle != 0
      DllCall(::nDllHandle,DLL_STDCALL,"SHFileOperationA",@::cFileOpStruct)
      DllUnload(::nDllHandle)
   ENDIF

   // Clean up BAP
   BaExtract(::aOpStruct)
   BaExtract(::aOpStruct)
   BaExtract(::aOpStruct)
   BaExtract(::aOpStruct)
   BaExtract(::aOpStruct)
   BaExtract(::aOpStruct)
   BaExtract(::aOpStruct)
   BaExtract(::aOpStruct)
   ::aOpStruct := NIL

RETURN NIL




/*
  Using SHFile -- Example
  PROC MAIN
LOCAL cSHFO, nH

   oSHFO := ShFile():New():Create()

   ? "Creating ShFile1.TXT"

   nH := FCREATE( "ShFile1.TXT" )
   FWrite( nH, "James Bond's BMW", 16 )
   FClose( nH )

   ? "Creating ShFile2.TXT"

   nH := FCREATE( "ShFile2.TXT" )
   FWrite( nH, "James Bond's new BMW", 16 )
   FClose( nH )

   ? "Creating a Directory"
   oSHFO:CreateDir( "SUB1" )

   nChoose := 1

   DO WHILE nChoose # 0

      @ 5,5 PROMPT " Delete files                        "
      @ 6,5 PROMPT " Copy file to single destination     "
      @ 7,5 PROMPT " Copy multiple files to multidest.   "
      @ 8,5 PROMPT " Rename file                         "
      @ 9,5 PROMPT " Rename multiple files               "

      MENU TO nChoose

      DO CASE
         CASE nChoose = 1
            oSHFO:Delete( { "ShFile1.TXT", "ShFile2.TXT" } )

         CASE nChoose = 2
            oSHFO:Copy( "ShFile1.TXT", "C:\WINDOWS\TEMP" )

         CASE nChoose = 3
            oSHFO:Copy( { "ShFile1.TXT", "ShFile2.TXT" }, { "C:\WINDOWS\TEMP\ShFile1.txt", "C:\TEMP\ShFile2.txt" } )

         CASE nChoose = 4
            oSHFO:Rename( "ShFile1.TXT", "NewShFile1.TXT" )

         CASE nChoose = 5
            oSHFO:Rename( "*.TXT", "*.TX2" )

         OTHERWISE
            nChoose = 0

      EndCASE

   ENDDO
   @ 11,5 SAY "Press any key to continue"
   oSHFO:RemoveDir( "SUB1" )

   INKEY(0)

RETURN




*/



******************************************************
* FUNCTION WinPopFolder
******************************************************

// This code requires the BAP Lib

#define NULL 0
#define BIF_RETURNONLYFSDIRS   0x0001  // For finding a folder to start document searching
#define BIF_DONTGOBELOWDOMAIN  0x0002  // For starting the Find Computer
#define BIF_STATUSTEXT         0x0004
#define BIF_RETURNFSANCESTORS  0x0008
#define BIF_EDITBOX            0x0010
#define BIF_VALIDATE           0x0020   // insist on valid result (or CANCEL)
#define BIF_BROWSEFORCOMPUTER  0x1000  // Browsing for Computers.
#define BIF_BROWSEFORPRINTER   0x2000  // Browsing for Printers
#define BIF_BROWSEINCLUDEFILES 0x4000  // Browsing for Everything

DLLFUNCTION SHBrowseForFolder( cStru ) USING STDCALL FROM SHELL32.DLL
DLLFUNCTION SHGetPathFromIDList(pidl,@cFolderName) USING STDCALL FROM SHELL32.DLL
DLLFUNCTION CoTaskMemFree(p) USING STDCALL FROM OLE32.DLL

FUNCTION WinPopFolder( oWin, cPrompt, lEditBox, lValidate, lFiles )

LOCAL hWnd      := IIF(oWIn=NIL,SetAppWindow():getHWND(),oWin:getHWND())
LOCAL aOFN
LOCAL cOFN
LOCAL i
LOCAL nFlags    := 0
LOCAL ret       := ''
LOCAL nIDL      := 0
LOCAL cFolderName := replicate(chr(0),260)


DEFAULT lEditBox TO .F.
DEFAULT lValidate TO .F.
DEFAULT lFiles TO .F.

cPrompt = ConvToAnsiCP(IIF(cPrompt=NIL,'',cPrompt)+CHR(0))
IF lEditBox
    nFlags := nFlags+BIF_EDITBOX
ENDIF
IF lValidate
    nFlags := nFlags+BIF_VALIDATE
ENDIF
IF lFiles
    nFlags := nFlags+BIF_BROWSEINCLUDEFILES
ENDIF

aOFN = BaInit(8)

BaStruct(aOFN,hWnd)                          // hwndOwner
BaStruct(aOFN,NULL)                          // pidlRoot

BaStruct(aOFN,@cFolderName)                  // lpstrFolder
BaStruct(aOFN,@cPrompt)                      // lpstrPrompt
BaStruct(aOFN,nFlags)                        // nFlags
BaStruct(aOFN,0)                             // CallBackProc
BaStruct(aOFN,0)                             // lParam
BaStruct(aOFN,0)                             // iImage

cOFN = BaAccess(aOFN)

nIDL = SHBrowseForFolder(cOFN)

IF nIDL > 0
   SHGetPathFromIDList(nIDL,@cFolderName)
   CoTaskMemFree(nIDL)
   ret = ConvToOEMCP(substr(cFolderName,1,at(chr(0),cFolderName)-1))
ENDIF

RETURN(ret)

***********************************************************************



***********************************************************************
* STATIC PROCEDURE EscKey()
***********************************************************************
STATIC PROCEDURE EscKey()

keybd_event( VK_ESCAPE, 0, 0, 0 )
keybd_event( VK_ESCAPE, 0, KEYEVENTF_KEYUP, 0 )

RETURN


